@@ -20,7 +20,7 @@ void TrieMapNode_Free(TrieMapNode *n, void (*freeCB)(void *));
 
 /* The byte size of a node, based on its internal string length and number of
  * children */
-tm_len_t __trieMapNode_Sizeof(tm_len_t numChildren, tm_len_t slen) {
+size_t __trieMapNode_Sizeof(tm_len_t numChildren, tm_len_t slen) {
   return (sizeof(TrieMapNode) + numChildren * sizeof(TrieMapNode *) + (slen + 1) + numChildren);
 }
 
@@ -220,45 +220,40 @@ void *TrieMapNode_Find(TrieMapNode *n, char *str, tm_len_t len) {
       // let's find a child to continue to
       tm_len_t i = 0;
       TrieMapNode *nextChild = NULL;
-      // if (!(n->flags & TM_NODE_SORTED) && n->numChildren > 10) {
-      //   qsort(__trieMapNode_children(n), n->numChildren, sizeof(TrieMapNode
-      //   *),
-      //         __cmp_nodes);
-      //   // printf("%.*s ... ", n->numChildren, __trieMapNode_childKey(n,
-      //   0));
-      //   qsort(__trieMapNode_childKey(n, 0), n->numChildren, 1,
-      //   __cmp_chars);
-      //   //  printf("%.*s\n", n->numChildren, __trieMapNode_childKey(n, 0));
+      // if (!(n->flags & TM_NODE_SORTED) && n->numChildren > 2) {
+      //   qsort(__trieMapNode_children(n), n->numChildren, sizeof(TrieMapNode *), __cmp_nodes);
+      //   qsort(__trieMapNode_childKey(n, 0), n->numChildren, 1, __cmp_chars);
       //   n->flags |= TM_NODE_SORTED;
       // }
       char *childKeys = __trieMapNode_childKey(n, 0);
       char c = str[offset];
-      if (n->flags & TM_NODE_SORTED) {
-        int bottom = 0, top = n->numChildren - 1;
+      // if (n->flags & TM_NODE_SORTED) {
+      //   int bottom = 0, top = n->numChildren - 1;
 
-        while (bottom <= top) {
-          int mid = (bottom + top) / 2;
+      //   while (bottom <= top) {
+      //     int mid = (bottom + top) / 2;
 
-          char cc = *__trieMapNode_childKey(n, mid);
-          if (c < cc) {
-            top = mid - 1;
-          } else if (c > cc) {
-            bottom = mid + 1;
-          } else {
-            nextChild = __trieMapNode_children(n)[mid];
-            break;
-          }
-        }
+      //     char cc = *__trieMapNode_childKey(n, mid);
+      //     if (c == cc) {
+      //       nextChild = __trieMapNode_children(n)[mid];
+      //       break;
+      //     } else if (c < cc) {
+      //       top = mid - 1;
+      //     } else {
+      //       bottom = mid + 1;
+      //     }
+      //   }
 
-      } else {
-        while (i < n->numChildren) {
-          if (str[offset] == childKeys[i]) {
-            nextChild = __trieMapNode_children(n)[i];
-            break;
-          }
-          ++i;
+      // } else {
+      tm_len_t nc = n->numChildren;
+      while (i < nc) {
+        if (str[offset] == childKeys[i]) {
+          nextChild = __trieMapNode_children(n)[i];
+          break;
         }
+        ++i;
       }
+      //}
 
       // we couldn't find a matching child
       n = nextChild;
@@ -358,9 +353,9 @@ TrieMapNode *__trieMapNode_MergeWithSingleChild(TrieMapNode *n) {
 }
 
 /* Optimize the node and its children:
-*   1. If a child should be deleted - delete it and reduce the child count
-*   2. If a child has a single child - merge them
-*/
+ *   1. If a child should be deleted - delete it and reduce the child count
+ *   2. If a child has a single child - merge them
+ */
 void __trieMapNode_optimizeChildren(TrieMapNode *n, void (*freeCB)(void *)) {
   int i = 0;
   TrieMapNode **nodes = __trieMapNode_children(n);
@@ -502,7 +497,7 @@ void TrieMapNode_Free(TrieMapNode *n, void (*freeCB)(void *)) {
 /* the current top of the iterator stack */
 #define __tmi_current(it) &it->stack[it->stackOffset - 1]
 
-/* Step itearator return codes below: */
+  /* Step itearator return codes below: */
 
 #define TM_ITERSTATE_SELF 0
 #define TM_ITERSTATE_CHILDREN 1
@@ -510,11 +505,19 @@ void TrieMapNode_Free(TrieMapNode *n, void (*freeCB)(void *)) {
 /* Push a new trie node on the iterator's stack */
 inline void __tmi_Push(TrieMapIterator *it, TrieMapNode *node) {
   if (it->stackOffset == it->stackCap) {
-    it->stackCap += MIN(it->stackCap, 1024);
+    // make sure we don't overflow stackCap
+    if ((int)it->stackCap + 1024 > 0xFFFF) {
+      it->stackCap = 0xFFFF;
+    } else {
+      it->stackCap += MIN(it->stackCap, 1024);
+    }
     it->stack = realloc(it->stack, it->stackCap * sizeof(__tmi_stackNode));
   }
   it->stack[it->stackOffset++] = (__tmi_stackNode){
-      .childOffset = 0, .stringOffset = 0, .n = node, .state = TM_ITERSTATE_SELF,
+      .childOffset = 0,
+      .stringOffset = 0,
+      .n = node,
+      .state = TM_ITERSTATE_SELF,
   };
 }
 
@@ -573,7 +576,11 @@ int TrieMapIterator_Next(TrieMapIterator *it, char **ptr, tm_len_t *len, void **
 
         // if needed - increase the buffer on the heap
         if (it->bufOffset == it->bufLen) {
-          it->bufLen *= 2;
+          if ((int)it->bufLen + 1024 > 0xFFFF) {
+            it->bufLen = 0xFFFF;
+          } else {
+            it->bufLen += MIN(it->bufLen, 1024);
+          }
           it->buf = realloc(it->buf, it->bufLen);
         }
       }
@@ -637,12 +644,17 @@ TrieMapNode *TrieMapNode_RandomWalk(TrieMapNode *n, int minSteps, char **str, tm
   TrieMapNode **stack = calloc(stackCap, sizeof(TrieMapNode *));
   stack[0] = n;
 
+  if (stackSz == stackCap) {
+    stackCap += minSteps;
+    stack = realloc(stack, stackCap * sizeof(TrieMapNode *));
+  }
+
   size_t bufCap = n->len;
 
   int steps = 0;
 
   while (steps < minSteps || !__trieMapNode_isTerminal(stack[stackSz - 1])) {
-    TrieMapNode *n = stack[stackSz - 1];
+    n = stack[stackSz - 1];
 
     /* select the next step - -1 means walk back up one level */
     int rnd = rand() % (n->numChildren + 1) - 1;
@@ -661,7 +673,7 @@ TrieMapNode *TrieMapNode_RandomWalk(TrieMapNode *n, int minSteps, char **str, tm
     steps++;
     if (stackSz == stackCap) {
       stackCap += minSteps;
-      stack = realloc(stack, stackCap);
+      stack = realloc(stack, stackCap * sizeof(TrieMapNode *));
     }
 
     bufCap += n->len;
