@@ -2,7 +2,9 @@
 #define __EXT_DEFAULT_H__
 #include "redisearch.h"
 
-#define DEFAULT_EXPANDER_NAME "SBSTEM"
+#define SYNONYMS_EXPENDER_NAME "SYNONYM"
+#define STEMMER_EXPENDER_NAME "SBSTEM"
+#define DEFAULT_EXPANDER_NAME "DEFAULT"
 #define DEFAULT_SCORER_NAME "TFIDF"
 #define TFIDF_DOCNORM_SCORER_NAME "TFIDF.DOCNORM"
 #define DISMAX_SCORER_NAME "DISMAX"
