@@ -17,6 +17,6 @@ typedef struct {
 /* Create a new IdListIterator from a pre populated list of document ids of size num. The doc ids
  * are sorted in this function, so there is no need to sort them. They are automatically freed in
  * the end and assumed to be allocated using rm_malloc */
-IndexIterator *NewIdListIterator(t_docId *ids, t_offset num);
+IndexIterator *NewIdListIterator(t_docId *ids, t_offset num, double weight);
 
 #endif
\ No newline at end of file
