@@ -11,13 +11,14 @@
 #include "query_parser/parser.h"
 #include "redis_index.h"
 #include "tokenize.h"
-#include "util/minmax_heap.h"
 #include "util/logging.h"
 #include "extension.h"
 #include "ext/default.h"
 #include "rmutil/sds.h"
 #include "tag_index.h"
+#include "err.h"
 #include "concurrent_ctx.h"
+#include "util/strconv.h"
 
 static void QueryTokenNode_Free(QueryTokenNode *tn) {
 
@@ -87,9 +88,13 @@ void QueryNode_Free(QueryNode *n) {
       if (n->gn.gf) {
         GeoFilter_Free(n->gn.gf);
       }
-
+      break;
+    case QN_FUZZY:
+      QueryTokenNode_Free(&n->fz.tok);
+      break;
     case QN_WILDCARD:
     case QN_IDS:
+
       break;
 
     case QN_TAG:
@@ -101,15 +106,19 @@ void QueryNode_Free(QueryNode *n) {
 static QueryNode *NewQueryNode(QueryNodeType type) {
   QueryNode *s = calloc(1, sizeof(QueryNode));
   s->type = type;
-  s->fieldMask = RS_FIELDMASK_ALL;
-  s->flags = 0;
+  s->opts = (QueryNodeOptions){
+      .fieldMask = RS_FIELDMASK_ALL,
+      .flags = 0,
+      .maxSlop = -1,
+      .inOrder = 0,
+      .weight = 1,
+  };
   return s;
 }
 
 QueryNode *NewTokenNodeExpanded(QueryParseCtx *q, const char *s, size_t len, RSTokenFlags flags) {
   QueryNode *ret = NewQueryNode(QN_TOKEN);
   q->numTokens++;
-
   ret->tn = (QueryTokenNode){.str = (char *)s, .len = len, .expanded = 1, .flags = flags};
   return ret;
 }
@@ -138,6 +147,23 @@ QueryNode *NewPrefixNode(QueryParseCtx *q, const char *s, size_t len) {
   return ret;
 }
 
+QueryNode *NewFuzzyNode(QueryParseCtx *q, const char *s, size_t len, int maxDist) {
+  QueryNode *ret = NewQueryNode(QN_FUZZY);
+  q->numTokens++;
+
+  ret->fz = (QueryFuzzyNode){
+      .tok =
+          (RSToken){
+              .str = (char *)s,
+              .len = len,
+              .expanded = 0,
+              .flags = 0,
+          },
+      .maxDist = maxDist,
+  };
+  return ret;
+}
+
 QueryNode *NewUnionNode() {
   QueryNode *ret = NewQueryNode(QN_UNION);
   // ret->fieldMask = 0;
@@ -235,7 +261,7 @@ static void QueryNode_Expand(RSQueryTokenExpander expander, RSQueryExpanderCtx *
 
   QueryNode *qn = *pqn;
   // Do not expand verbatim nodes
-  if (qn->flags & QueryNode_Verbatim) {
+  if (qn->opts.flags & QueryNode_Verbatim) {
     return;
   }
 
@@ -257,7 +283,8 @@ static void QueryNode_Expand(RSQueryTokenExpander expander, RSQueryExpanderCtx *
 void Query_Expand(QueryParseCtx *q, const char *expander) {
   if (!q->root) return;
 
-  RSQueryExpanderCtx expCtx = {.query = q, .language = q->language};
+  RSQueryExpanderCtx expCtx = {.query = q,
+                               .language = q->opts.language ? q->opts.language : DEFAULT_LANGUAGE};
 
   ExtQueryExpanderCtx *xpc =
       Extensions_GetQueryExpander(&expCtx, expander ? expander : DEFAULT_EXPANDER_NAME);
@@ -274,12 +301,13 @@ IndexIterator *Query_EvalTokenNode(QueryEvalCtx *q, QueryNode *qn) {
   // if there's only one word in the query and no special field filtering,
   // and we are not paging beyond MAX_SCOREINDEX_SIZE
   // we can just use the optimized score index
-  int isSingleWord = q->numTokens == 1 && q->req->fieldMask == RS_FIELDMASK_ALL;
+  int isSingleWord = q->numTokens == 1 && q->opts->fieldMask == RS_FIELDMASK_ALL;
 
   RSQueryTerm *term = NewQueryTerm(&qn->tn, q->tokenId++);
 
-  IndexReader *ir = Redis_OpenReader(q->sctx, term, q->docTable, isSingleWord,
-                                     q->req->fieldMask & qn->fieldMask, q->conc);
+  IndexReader *ir =
+      Redis_OpenReader(q->sctx, term, q->docTable, isSingleWord,
+                       q->opts->fieldMask & qn->opts.fieldMask, q->conc, qn->opts.weight);
   if (ir == NULL) {
     Term_Free(term);
     return NULL;
@@ -288,22 +316,10 @@ IndexIterator *Query_EvalTokenNode(QueryEvalCtx *q, QueryNode *qn) {
   return NewReadIterator(ir);
 }
 
-/* Ealuate a prefix node by expanding all its possible matches and creating one big UNION on all of
- * them */
-static IndexIterator *Query_EvalPrefixNode(QueryEvalCtx *q, QueryNode *qn) {
-  if (qn->type != QN_PREFX) {
-    return NULL;
-  }
-
-  // we allow a minimum of 2 letters in the prefx by default (configurable)
-  if (qn->pfx.len < RSGlobalConfig.minTermPrefix) {
-    return NULL;
-  }
-  Trie *terms = q->sctx->spec->terms;
-
-  if (!terms) return NULL;
-
-  TrieIterator *it = Trie_IteratePrefix(terms, qn->pfx.str, qn->pfx.len, 0);
+static IndexIterator *iterateExpandedTerms(QueryEvalCtx *q, Trie *terms, const char *str,
+                                           size_t len, int maxDist, int prefixMode,
+                                           QueryNodeOptions *opts) {
+  TrieIterator *it = Trie_Iterate(terms, str, len, maxDist, prefixMode);
   if (!it) return NULL;
 
   size_t itsSz = 0, itsCap = 8;
@@ -326,11 +342,15 @@ static IndexIterator *Query_EvalPrefixNode(QueryEvalCtx *q, QueryNode *qn) {
         .len = 0,
     };
     tok.str = runesToStr(rstr, slen, &tok.len);
+    if (q->sctx && q->sctx->redisCtx) {
+      RedisModule_Log(q->sctx->redisCtx, "debug", "Found fuzzy expansion: %s %f", tok.str, score);
+    }
+
     RSQueryTerm *term = NewQueryTerm(&tok, q->tokenId++);
 
     // Open an index reader
     IndexReader *ir = Redis_OpenReader(q->sctx, term, &q->sctx->spec->docs, 0,
-                                       q->req->fieldMask & qn->fieldMask, q->conc);
+                                       q->opts->fieldMask & opts->fieldMask, q->conc, 1);
 
     free(tok.str);
     if (!ir) {
@@ -354,7 +374,32 @@ static IndexIterator *Query_EvalPrefixNode(QueryEvalCtx *q, QueryNode *qn) {
     free(its);
     return NULL;
   }
-  return NewUnionIterator(its, itsSz, q->docTable, 1);
+  return NewUnionIterator(its, itsSz, q->docTable, 1, opts->weight);
+}
+/* Ealuate a prefix node by expanding all its possible matches and creating one big UNION on all
+ * of them */
+static IndexIterator *Query_EvalPrefixNode(QueryEvalCtx *q, QueryNode *qn) {
+  assert(qn->type == QN_PREFX);
+
+  // we allow a minimum of 2 letters in the prefx by default (configurable)
+  if (qn->pfx.len < RSGlobalConfig.minTermPrefix) {
+    return NULL;
+  }
+  Trie *terms = q->sctx->spec->terms;
+
+  if (!terms) return NULL;
+
+  return iterateExpandedTerms(q, terms, qn->pfx.str, qn->pfx.len, 0, 1, &qn->opts);
+}
+
+static IndexIterator *Query_EvalFuzzyNode(QueryEvalCtx *q, QueryNode *qn) {
+  assert(qn->type == QN_FUZZY);
+
+  Trie *terms = q->sctx->spec->terms;
+
+  if (!terms) return NULL;
+
+  return iterateExpandedTerms(q, terms, qn->pfx.str, qn->pfx.len, qn->fz.maxDist, 0, &qn->opts);
 }
 
 static IndexIterator *Query_EvalPhraseNode(QueryEvalCtx *q, QueryNode *qn) {
@@ -365,26 +410,39 @@ static IndexIterator *Query_EvalPhraseNode(QueryEvalCtx *q, QueryNode *qn) {
   // an intersect stage with one child is the same as the child, so we just
   // return it
   if (node->numChildren == 1) {
-    node->children[0]->fieldMask &= qn->fieldMask;
+    node->children[0]->opts.fieldMask &= qn->opts.fieldMask;
     return Query_EvalNode(q, node->children[0]);
   }
 
   // recursively eval the children
   IndexIterator **iters = calloc(node->numChildren, sizeof(IndexIterator *));
   for (int i = 0; i < node->numChildren; i++) {
-    node->children[i]->fieldMask &= qn->fieldMask;
+    node->children[i]->opts.fieldMask &= qn->opts.fieldMask;
     iters[i] = Query_EvalNode(q, node->children[i]);
   }
   IndexIterator *ret;
 
   if (node->exact) {
     ret = NewIntersecIterator(iters, node->numChildren, q->docTable,
-                              q->req->fieldMask & qn->fieldMask, 0, 1);
+                              q->opts->fieldMask & qn->opts.fieldMask, 0, 1, qn->opts.weight);
   } else {
-    ret = NewIntersecIterator(iters, node->numChildren, q->docTable,
+    // Let the query node override the slop/order parameters
+    int slop = qn->opts.maxSlop;
+    if (slop == -1) slop = q->opts->slop;
 
-                              q->req->fieldMask & qn->fieldMask, q->req->slop,
-                              q->req->flags & Search_InOrder);
+    // Let the query node override the inorder of the whole query
+    int inOrder = q->opts->flags & Search_InOrder;
+    if (qn->opts.inOrder) inOrder = 1;
+
+    // If in order was specified and not slop, set slop to maximum possible value.
+    // Otherwise we can't check if the results are in order
+    if (inOrder && slop == -1) {
+      slop = __INT_MAX__;
+    }
+
+    ret = NewIntersecIterator(iters, node->numChildren, q->docTable,
+                              q->opts->fieldMask & qn->opts.fieldMask, slop, inOrder,
+                              qn->opts.weight);
   }
   return ret;
 }
@@ -403,7 +461,8 @@ static IndexIterator *Query_EvalNotNode(QueryEvalCtx *q, QueryNode *qn) {
   }
   QueryNotNode *node = &qn->not;
 
-  return NewNotIterator(node->child ? Query_EvalNode(q, node->child) : NULL, q->docTable->maxDocId);
+  return NewNotIterator(node->child ? Query_EvalNode(q, node->child) : NULL, q->docTable->maxDocId,
+                        qn->opts.weight);
 }
 
 static IndexIterator *Query_EvalOptionalNode(QueryEvalCtx *q, QueryNode *qn) {
@@ -413,7 +472,7 @@ static IndexIterator *Query_EvalOptionalNode(QueryEvalCtx *q, QueryNode *qn) {
   QueryOptionalNode *node = &qn->opt;
 
   return NewOptionalIterator(node->child ? Query_EvalNode(q, node->child) : NULL,
-                             q->docTable->maxDocId);
+                             q->docTable->maxDocId, qn->opts.weight);
 }
 
 static IndexIterator *Query_EvalNumericNode(QueryEvalCtx *q, QueryNumericNode *node) {
@@ -427,7 +486,8 @@ static IndexIterator *Query_EvalNumericNode(QueryEvalCtx *q, QueryNumericNode *n
   return NewNumericFilterIterator(q->sctx, node->nf, q->conc);
 }
 
-static IndexIterator *Query_EvalGeofilterNode(QueryEvalCtx *q, QueryGeofilterNode *node) {
+static IndexIterator *Query_EvalGeofilterNode(QueryEvalCtx *q, QueryGeofilterNode *node,
+                                              double weight) {
 
   FieldSpec *fs = IndexSpec_GetField(q->sctx->spec, node->gf->property, strlen(node->gf->property));
   if (fs == NULL || fs->type != FIELD_GEO) {
@@ -435,7 +495,7 @@ static IndexIterator *Query_EvalGeofilterNode(QueryEvalCtx *q, QueryGeofilterNod
   }
 
   GeoIndex gi = {.ctx = q->sctx, .sp = fs};
-  return NewGeoRangeIterator(&gi, node->gf);
+  return NewGeoRangeIterator(&gi, node->gf, weight);
 }
 
 static IndexIterator *Query_EvalIdFilterNode(QueryEvalCtx *q, QueryIdFilterNode *node) {
@@ -451,7 +511,7 @@ static IndexIterator *Query_EvalUnionNode(QueryEvalCtx *q, QueryNode *qn) {
 
   // a union stage with one child is the same as the child, so we just return it
   if (node->numChildren == 1) {
-    node->children[0]->fieldMask &= qn->fieldMask;
+    node->children[0]->opts.fieldMask &= qn->opts.fieldMask;
     return Query_EvalNode(q, node->children[0]);
   }
 
@@ -459,7 +519,7 @@ static IndexIterator *Query_EvalUnionNode(QueryEvalCtx *q, QueryNode *qn) {
   IndexIterator **iters = calloc(node->numChildren, sizeof(IndexIterator *));
   int n = 0;
   for (int i = 0; i < node->numChildren; i++) {
-    node->children[i]->fieldMask &= qn->fieldMask;
+    node->children[i]->opts.fieldMask &= qn->opts.fieldMask;
     IndexIterator *it = Query_EvalNode(q, node->children[i]);
     if (it) {
       iters[n++] = it;
@@ -476,19 +536,69 @@ static IndexIterator *Query_EvalUnionNode(QueryEvalCtx *q, QueryNode *qn) {
     return ret;
   }
 
-  IndexIterator *ret = NewUnionIterator(iters, n, q->docTable, 0);
+  IndexIterator *ret = NewUnionIterator(iters, n, q->docTable, 0, qn->opts.weight);
   return ret;
 }
 
+/* Evaluate a tag prefix by expanding it with a lookup on the tag index */
+static IndexIterator *Query_EvalTagPrefixNode(QueryEvalCtx *q, TagIndex *idx, QueryNode *qn,
+                                              RedisModuleKey *k, RedisModuleString *kn,
+                                              double weight) {
+  if (qn->type != QN_PREFX) {
+    return NULL;
+  }
+
+  // we allow a minimum of 2 letters in the prefx by default (configurable)
+  if (qn->pfx.len < RSGlobalConfig.minTermPrefix) {
+    return NULL;
+  }
+  if (!idx || !idx->values) return NULL;
+
+  TrieMapIterator *it = TrieMap_Iterate(idx->values, qn->pfx.str, qn->pfx.len);
+  if (!it) return NULL;
+
+  size_t itsSz = 0, itsCap = 8;
+  IndexIterator **its = calloc(itsCap, sizeof(*its));
+
+  // an upper limit on the number of expansions is enforced to avoid stuff like "*"
+  char *s;
+  tm_len_t sl;
+  void *ptr;
+
+  // Find all completions of the prefix
+  while (TrieMapIterator_Next(it, &s, &sl, &ptr) && itsSz < RSGlobalConfig.maxPrefixExpansions) {
+    IndexIterator *ret = TagIndex_OpenReader(idx, q->docTable, s, sl, q->conc, k, kn, 1);
+    if (!ret) continue;
+
+    // Add the reader to the iterator array
+    its[itsSz++] = ret;
+    if (itsSz == itsCap) {
+      itsCap *= 2;
+      its = realloc(its, itsCap * sizeof(*its));
+    }
+  }
+
+  TrieMapIterator_Free(it);
+
+  // printf("Expanded %d terms!\n", itsSz);
+  if (itsSz == 0) {
+    free(its);
+    return NULL;
+  }
+  return NewUnionIterator(its, itsSz, q->docTable, 1, weight);
+}
+
 static IndexIterator *query_EvalSingleTagNode(QueryEvalCtx *q, TagIndex *idx, QueryNode *n,
-                                              RedisModuleKey *k, RedisModuleString *kn) {
+                                              RedisModuleKey *k, RedisModuleString *kn,
+                                              double weight) {
 
   switch (n->type) {
     case QN_TOKEN:
-      /*TagIndex *idx, DocTable *dt, const char *value, size_t len,
-                                         ConcurrentSearchCtx *csx, RedisModuleKey *k,
-                                         RedisModuleString *keyName);*/
-      return TagIndex_OpenReader(idx, q->docTable, n->tn.str, n->tn.len, q->conc, k, kn);
+
+      return TagIndex_OpenReader(idx, q->docTable, n->tn.str, n->tn.len, q->conc, k, kn, weight);
+    case QN_PREFX:
+      return Query_EvalTagPrefixNode(q, idx, n, k, kn, weight);
+
     case QN_PHRASE: {
       char *terms[n->pn.numChildren];
       for (int i = 0; i < n->pn.numChildren; i++) {
@@ -501,7 +611,8 @@ static IndexIterator *query_EvalSingleTagNode(QueryEvalCtx *q, TagIndex *idx, Qu
 
       sds s = sdsjoin(terms, n->pn.numChildren, " ");
 
-      IndexIterator *ret = TagIndex_OpenReader(idx, q->docTable, s, sdslen(s), q->conc, k, kn);
+      IndexIterator *ret =
+          TagIndex_OpenReader(idx, q->docTable, s, sdslen(s), q->conc, k, kn, weight);
       sdsfree(s);
       return ret;
     }
@@ -520,17 +631,16 @@ static IndexIterator *Query_EvalTagNode(QueryEvalCtx *q, QueryNode *qn) {
   RedisModuleString *str = TagIndex_FormatName(q->sctx, node->fieldName);
   TagIndex *idx = TagIndex_Open(q->sctx->redisCtx, str, 0, &k);
   if (!idx) return NULL;
-
   // a union stage with one child is the same as the child, so we just return it
   if (node->numChildren == 1) {
-    return query_EvalSingleTagNode(q, idx, node->children[0], k, str);
+    return query_EvalSingleTagNode(q, idx, node->children[0], k, str, qn->opts.weight);
   }
 
   // recursively eval the children
   IndexIterator **iters = calloc(node->numChildren, sizeof(IndexIterator *));
   int n = 0;
   for (int i = 0; i < node->numChildren; i++) {
-    IndexIterator *it = query_EvalSingleTagNode(q, idx, node->children[i], k, str);
+    IndexIterator *it = query_EvalSingleTagNode(q, idx, node->children[i], k, str, qn->opts.weight);
     if (it) {
       iters[n++] = it;
     }
@@ -540,7 +650,7 @@ static IndexIterator *Query_EvalTagNode(QueryEvalCtx *q, QueryNode *qn) {
     return NULL;
   }
 
-  IndexIterator *ret = NewUnionIterator(iters, n, q->docTable, 0);
+  IndexIterator *ret = NewUnionIterator(iters, n, q->docTable, 0, qn->opts.weight);
   return ret;
 }
 
@@ -558,12 +668,14 @@ IndexIterator *Query_EvalNode(QueryEvalCtx *q, QueryNode *n) {
       return Query_EvalNotNode(q, n);
     case QN_PREFX:
       return Query_EvalPrefixNode(q, n);
+    case QN_FUZZY:
+      return Query_EvalFuzzyNode(q, n);
     case QN_NUMERIC:
       return Query_EvalNumericNode(q, &n->nn);
     case QN_OPTIONAL:
       return Query_EvalOptionalNode(q, n);
     case QN_GEO:
-      return Query_EvalGeofilterNode(q, &n->gn);
+      return Query_EvalGeofilterNode(q, &n->gn, n->opts.weight);
     case QN_IDS:
       return Query_EvalIdFilterNode(q, &n->fn);
     case QN_WILDCARD:
@@ -573,11 +685,11 @@ IndexIterator *Query_EvalNode(QueryEvalCtx *q, QueryNode *n) {
   return NULL;
 }
 
-/* Set the field mask recursively on a query node. This is called by the parser to handle situations
- * like @foo:(bar baz|gaz), where a complex tree is being applied a field mask */
+/* Set the field mask recursively on a query node. This is called by the parser to handle
+ * situations like @foo:(bar baz|gaz), where a complex tree is being applied a field mask */
 void QueryNode_SetFieldMask(QueryNode *n, t_fieldMask mask) {
   if (!n) return;
-  n->fieldMask &= mask;
+  n->opts.fieldMask &= mask;
   switch (n->type) {
 
     case QN_PHRASE:
@@ -626,32 +738,42 @@ void QueryTagNode_AddChildren(QueryNode *parent, QueryNode **children, size_t nu
 
   tn->children = realloc(tn->children, sizeof(QueryNode *) * (tn->numChildren + num));
   for (size_t i = 0; i < num; i++) {
-    if (children[i] && (children[i]->type == QN_TOKEN || children[i]->type == QN_PHRASE)) {
+    if (children[i] && (children[i]->type == QN_TOKEN || children[i]->type == QN_PHRASE ||
+                        children[i]->type == QN_PREFX)) {
       tn->children[tn->numChildren++] = children[i];
     }
   }
 }
 
-QueryParseCtx *NewQueryParseCtx(RSSearchRequest *req) {
+static void assignSearchOpts(RSSearchOptions *tgt, const RSSearchOptions *src,
+                             RedisSearchCtx *ctx) {
+  if (src) {
+    *tgt = *src;
+  } else {
+    *tgt = RS_DEFAULT_SEARCHOPTS;
+  }
+  if (tgt->flags & Search_NoStopwrods) {
+    tgt->stopwords = EmptyStopWordList();
+  } else {
+    tgt->stopwords =
+        (ctx && ctx->spec && ctx->spec->stopwords) ? ctx->spec->stopwords : DefaultStopWordList();
+  }
+}
+
+QueryParseCtx *NewQueryParseCtx(RedisSearchCtx *sctx, const char *raw, size_t len,
+                                RSSearchOptions *opts) {
 
   QueryParseCtx *ctx = malloc(sizeof(*ctx));
-  ctx->len = req->qlen;
-  ctx->raw = strdup(req->rawQuery);
+  ctx->len = len;
+  ctx->raw = strdup(raw);
   ctx->numTokens = 0;
   ctx->ok = 1;
   ctx->root = NULL;
-  ctx->sctx = req->sctx;
-  if (req->flags & Search_NoStopwords) {
-    ctx->stopwords = EmptyStopWordList();
-  } else {
-    ctx->stopwords = (ctx->sctx && ctx->sctx->spec && ctx->sctx->spec->stopwords)
-                         ? ctx->sctx->spec->stopwords
-                         : DefaultStopWordList();
-  }
-  ctx->language = req->language ? req->language : DEFAULT_LANGUAGE;
+  ctx->sctx = sctx;
   ctx->tokenId = 1;
   ctx->errorMsg = NULL;
-  ctx->payloadptr = &req->payload;
+  assignSearchOpts(&ctx->opts, opts, sctx);
+
   return ctx;
 }
 
@@ -673,17 +795,17 @@ static sds doPad(sds s, int len) {
 static sds QueryNode_DumpSds(sds s, QueryParseCtx *q, QueryNode *qs, int depth) {
   s = doPad(s, depth);
 
-  if (qs->fieldMask == 0) {
+  if (qs->opts.fieldMask == 0) {
     s = sdscat(s, "@NULL:");
   }
 
-  if (qs->fieldMask && qs->fieldMask != RS_FIELDMASK_ALL && qs->type != QN_NUMERIC &&
+  if (qs->opts.fieldMask && qs->opts.fieldMask != RS_FIELDMASK_ALL && qs->type != QN_NUMERIC &&
       qs->type != QN_GEO && qs->type != QN_IDS) {
     if (!q || !q->sctx->spec) {
-      s = sdscatprintf(s, "@%" PRIu64, (uint64_t)qs->fieldMask);
+      s = sdscatprintf(s, "@%" PRIu64, (uint64_t)qs->opts.fieldMask);
     } else {
       s = sdscat(s, "@");
-      t_fieldMask fm = qs->fieldMask;
+      t_fieldMask fm = qs->opts.fieldMask;
       int i = 0, n = 0;
       while (fm) {
         t_fieldMask bit = (fm & 1) << i;
@@ -709,7 +831,11 @@ static sds QueryNode_DumpSds(sds s, QueryParseCtx *q, QueryNode *qs, int depth)
 
       break;
     case QN_TOKEN:
-      s = sdscatprintf(s, "%s%s\n", (char *)qs->tn.str, qs->tn.expanded ? "(expanded)" : "");
+      s = sdscatprintf(s, "%s%s", (char *)qs->tn.str, qs->tn.expanded ? "(expanded)" : "");
+      if (qs->opts.weight != 1) {
+        s = sdscatprintf(s, " => {$weight: %g;}", qs->opts.weight);
+      }
+      s = sdscat(s, "\n");
       return s;
 
     case QN_PREFX:
@@ -756,20 +882,39 @@ static sds QueryNode_DumpSds(sds s, QueryParseCtx *q, QueryNode *qs, int depth)
 
       s = sdscat(s, "IDS { ");
       for (int i = 0; i < qs->fn.f->size; i++) {
-        s = sdscatprintf(s, "%d,", qs->fn.f->ids[i]);
+        s = sdscatprintf(s, "%llu,", (unsigned long long)qs->fn.f->ids[i]);
       }
       break;
     case QN_WILDCARD:
 
       s = sdscat(s, "<WILDCARD>");
       break;
+    case QN_FUZZY:
+      s = sdscatprintf(s, "FUZZY{%s}\n", qs->fz.tok.str);
+      return s;
   }
 
-  s = sdscat(s, "}\n");
+  s = sdscat(s, "}");
+  // print attributes if not the default
+  if (qs->opts.weight != 1 || qs->opts.maxSlop != -1 || qs->opts.inOrder) {
+    s = sdscat(s, " => {");
+    if (qs->opts.weight != 1) {
+      s = sdscatprintf(s, " $weight: %g;", qs->opts.weight);
+    }
+    if (qs->opts.maxSlop != -1) {
+      s = sdscatprintf(s, " $slop: %d;", qs->opts.maxSlop);
+    }
+    if (qs->opts.inOrder || qs->opts.maxSlop != -1) {
+      s = sdscatprintf(s, " $inorder: %s;", qs->opts.inOrder ? "true" : "false");
+    }
+    s = sdscat(s, " }");
+  }
+  s = sdscat(s, "\n");
   return s;
 }
 
-/* Return a string representation of the query parse tree. The string should be freed by the caller
+/* Return a string representation of the query parse tree. The string should be freed by the
+ * caller
  */
 const char *Query_DumpExplain(QueryParseCtx *q) {
   // empty query
@@ -798,180 +943,49 @@ void Query_Free(QueryParseCtx *q) {
   free(q);
 }
 
-/******************************************************************************************************
- *   Query Plan - the actual binding context of the whole execution plan - from filters to
- *   processors
- ******************************************************************************************************/
+int QueryNode_ApplyAttribute(QueryNode *qn, QueryAttribute *attr, char **err) {
 
-static size_t serializeResult(QueryPlan *qex, SearchResult *r, RSSearchFlags flags) {
-  size_t count = 1;
+  // Apply slop: [-1 ... INF]
+  if (STR_EQCASE(attr->name, attr->namelen, "slop")) {
 
-  RedisModuleCtx *ctx = qex->ctx->redisCtx;
-  size_t keyLen;
-  const char *keyStr = DMD_KeyPtrLen(r->md, &keyLen);
-  RedisModule_ReplyWithStringBuffer(ctx, keyStr, keyLen);
+    long long n;
+    if (!ParseInteger(attr->value, &n) || n < -1) {
+      SET_ERR(err, "Invalid value for slop");
+      return 0;
+    }
+    qn->opts.maxSlop = n;
 
-  if (flags & Search_WithScores) {
-    RedisModule_ReplyWithDouble(ctx, r->score);
-    count++;
+  } else if (STR_EQCASE(attr->name, attr->namelen, "inorder")) {
+    // Apply inorder: true|false
+    int b;
+    if (!ParseBoolean(attr->value, &b)) {
+      SET_ERR(err, "Invalid value for 'inorder'");
+      return 0;
+    }
+    qn->opts.inOrder = b;
+
+  } else if (STR_EQCASE(attr->name, attr->namelen, "weight")) {
+    // Apply weight: [0  ... INF]
+    double d;
+    if (!ParseDouble(attr->value, &d) || d < 0) {
+      SET_ERR(err, "Invalid value for 'weight'");
+      return 0;
+    }
+    qn->opts.weight = d;
+
+  } else {
+    FMT_ERR(err, "Invalid attribute '%.*s'", (int)attr->namelen, attr->name);
+    return 0;
   }
 
-  if (flags & Search_WithPayloads) {
-    ++count;
-    const RSPayload *payload = r->md->payload;
-    if (payload) {
-      RedisModule_ReplyWithStringBuffer(ctx, payload->data, payload->len);
-    } else {
-      RedisModule_ReplyWithNull(ctx);
-    }
-  }
-
-  if (flags & Search_WithSortKeys) {
-    ++count;
-    const RSSortableValue *sortkey = RSSortingVector_Get(r->sv, qex->req->sortBy);
-    if (sortkey) {
-      if (sortkey->type == RS_SORTABLE_NUM) {
-        /* Serialize double - by prepending "%" to the number, so the coordinator/client can tell
-         * it's a double and not just a numeric string value */
-        RedisModule_ReplyWithString(ctx,
-                                    RedisModule_CreateStringPrintf(ctx, "#%.17g", sortkey->num));
-      } else if (sortkey->type == RS_SORTABLE_STR) {
-        /* Serialize string - by prepending "$" to it */
-        RedisModule_ReplyWithString(ctx, RedisModule_CreateStringPrintf(ctx, "$%s", sortkey->str));
-      } else {
-        // NIL, or any other type:
-        RedisModule_ReplyWithNull(ctx);
-      }
-    }
-
-    else {
-      RedisModule_ReplyWithNull(ctx);
-    }
-  }
-
-  if (!(flags & Search_NoContent)) {
-    count++;
-    RedisModule_ReplyWithArray(ctx, r->fields->len * 2);
-    for (int i = 0; i < r->fields->len; i++) {
-      RedisModule_ReplyWithStringBuffer(ctx, r->fields->fields[i].key,
-                                        strlen(r->fields->fields[i].key));
-      RSValue_SendReply(ctx, RSFieldMap_Item(r->fields, i));
-    }
-  }
-  return count;
+  return 1;
 }
 
-int Query_SerializeResults(QueryPlan *qex, RSSearchFlags flags) {
-  int rc;
-  int count = 0;
-  RedisModuleCtx *ctx = qex->ctx->redisCtx;
-  RedisModule_ReplyWithArray(ctx, REDISMODULE_POSTPONED_ARRAY_LEN);
-
-  do {
-    SearchResult r;
-    rc = ResultProcessor_Next(qex->rootProcessor, &r, 1);
-    if (rc == RS_RESULT_EOF) break;
-
-    if (count == 0) {
-      RedisModule_ReplyWithLongLong(ctx, ResultProcessor_Total(qex->rootProcessor));
-      count++;
-    }
-    count += serializeResult(qex, &r, flags);
-
-    IndexResult_Free(r.indexResult);
-    RSFieldMap_Free(r.fields, 0);
-  } while (rc != RS_RESULT_EOF);
-  if (count == 0) {
-    RedisModule_ReplyWithLongLong(ctx, ResultProcessor_Total(qex->rootProcessor));
-    count++;
-  }
-  RedisModule_ReplySetArrayLength(ctx, count);
-  return REDISMODULE_OK;
-}
-
-/* A callback called when we regain concurrent execution context, and the index spec key is
- * reopened. We protect against the case that the spec has been deleted during query execution */
-void Query_OnReopen(RedisModuleKey *k, void *privdata) {
-
-  IndexSpec *sp = RedisModule_ModuleTypeGetValue(k);
-  QueryPlan *q = privdata;
-
-  // If we don't have a spec or key - we abort the query
-  if (k == NULL || sp == NULL) {
-
-    q->execCtx.state = QueryState_Aborted;
-    q->ctx->spec = NULL;
-    return;
-  }
-
-  // The spec might have changed while we were sleeping - for example a realloc of the doc table
-  q->ctx->spec = sp;
-
-  if (RSGlobalConfig.queryTimeoutMS > 0) {
-    // Check the elapsed processing time
-    static struct timespec now;
-    clock_gettime(CLOCK_MONOTONIC_RAW, &now);
-
-    long long durationNS = (long long)1000000000 * (now.tv_sec - q->execCtx.startTime.tv_sec) +
-                           (now.tv_nsec - q->execCtx.startTime.tv_nsec);
-    // printf("Elapsed: %zdms\n", durationNS / 1000000);
-    // Abort on timeout
-    if (durationNS > RSGlobalConfig.queryTimeoutMS * 1000000) {
-      q->execCtx.state = QueryState_TimedOut;
+int QueryNode_ApplyAttributes(QueryNode *qn, QueryAttribute *attrs, size_t len, char **err) {
+  for (size_t i = 0; i < len; i++) {
+    if (!QueryNode_ApplyAttribute(qn, &attrs[i], err)) {
+      return 0;
     }
   }
-  // q->docTable = &sp->docs;
-}
-
-int QueryPlan_Execute(QueryPlan *plan, const char **err) {
-  int rc = Query_SerializeResults(plan, plan->req->flags);
-  if (err) *err = plan->execCtx.errorString;
-  return rc;
-}
-
-void QueryPlan_Free(QueryPlan *plan) {
-  if (plan->rootProcessor) {
-    ResultProcessor_Free(plan->rootProcessor);
-  }
-  if (plan->rootFilter) {
-    plan->rootFilter->Free(plan->rootFilter);
-  }
-  if (plan->conc) {
-    ConcurrentSearchCtx_Free(plan->conc);
-    free(plan->conc);
-  }
-  free(plan);
-}
-
-QueryPlan *Query_BuildPlan(QueryParseCtx *parsedQuery, RSSearchRequest *req, int concurrentMode) {
-  QueryPlan *plan = calloc(1, sizeof(*plan));
-  plan->ctx = req->sctx;
-  plan->conc = concurrentMode ? malloc(sizeof(*plan->conc)) : NULL;
-  plan->req = req;
-  plan->execCtx = (QueryProcessingCtx){.errorString = NULL,
-                                       .minScore = 0,
-                                       .totalResults = 0,
-                                       .state = QueryState_OK,
-                                       .sctx = plan->ctx,
-                                       .conc = plan->conc};
-  clock_gettime(CLOCK_MONOTONIC_RAW, &plan->execCtx.startTime);
-  if (plan->conc) {
-    ConcurrentSearchCtx_Init(req->sctx->redisCtx, plan->conc);
-    ConcurrentSearch_AddKey(plan->conc, plan->ctx->key, REDISMODULE_READ, plan->ctx->keyName,
-                            Query_OnReopen, plan, NULL, 0);
-  }
-
-  QueryEvalCtx ev = {
-      .docTable = plan->ctx && plan->ctx->spec ? &plan->ctx->spec->docs : NULL,
-      .conc = plan->conc,
-      .numTokens = parsedQuery->numTokens,
-      .tokenId = 1,
-      .sctx = plan->ctx,
-      .req = req,
-  };
-
-  plan->rootFilter = Query_EvalNode(&ev, parsedQuery->root);
-  plan->rootProcessor = Query_BuildProcessorChain(plan, req);
-  plan->execCtx.rootFilter = plan->rootFilter;
-  return plan;
-}
+  return 1;
+}
\ No newline at end of file
