@@ -5,6 +5,7 @@
 #include "rmutil/strings.h"
 #include "rmutil/util.h"
 #include "util/logging.h"
+#include "util/misc.h"
 #include "tag_index.h"
 #include "rmalloc.h"
 #include <stdio.h>
@@ -61,10 +62,6 @@ void InvertedIndex_RdbSave(RedisModuleIO *rdb, void *value) {
 }
 void InvertedIndex_Digest(RedisModuleDigest *digest, void *value) {
 }
-void InvertedIndex_AofRewrite(RedisModuleIO *aof, RedisModuleString *key, void *value) {
-  // TODO: Write something more optimal
-  RMUtil_DefaultAofRewrite(aof, key, value);
-}
 
 unsigned long InvertedIndex_MemUsage(const void *value) {
   const InvertedIndex *idx = value;
@@ -81,7 +78,7 @@ int InvertedIndex_RegisterType(RedisModuleCtx *ctx) {
   RedisModuleTypeMethods tm = {.version = REDISMODULE_TYPE_METHOD_VERSION,
                                .rdb_load = InvertedIndex_RdbLoad,
                                .rdb_save = InvertedIndex_RdbSave,
-                               .aof_rewrite = InvertedIndex_AofRewrite,
+                               .aof_rewrite = GenericAofRewrite_DisabledHandler,
                                .mem_usage = InvertedIndex_MemUsage,
                                .free = InvertedIndex_Free};
 
@@ -131,10 +128,9 @@ RedisModuleString *fmtRedisScoreIndexKey(RedisSearchCtx *ctx, const char *term,
                                         term);
 }
 
-RedisSearchCtx *NewSearchCtx(RedisModuleCtx *ctx, RedisModuleString *indexName) {
+RedisSearchCtx *NewSearchCtxC(RedisModuleCtx *ctx, const char *indexName) {
 
-  RedisModuleString *keyName = RedisModule_CreateStringPrintf(
-      ctx, INDEX_SPEC_KEY_FMT, RedisModule_StringPtrLen(indexName, NULL));
+  RedisModuleString *keyName = RedisModule_CreateStringPrintf(ctx, INDEX_SPEC_KEY_FMT, indexName);
 
   RedisModuleKey *k = RedisModule_OpenKey(ctx, keyName, REDISMODULE_READ);
   // printf("open key %s: %p\n", RedisModule_StringPtrLen(keyName, NULL), k);
@@ -154,6 +150,30 @@ RedisSearchCtx *NewSearchCtx(RedisModuleCtx *ctx, RedisModuleString *indexName)
   return sctx;
 }
 
+RedisSearchCtx *NewSearchCtx(RedisModuleCtx *ctx, RedisModuleString *indexName) {
+
+  return NewSearchCtxC(ctx, RedisModule_StringPtrLen(indexName, NULL));
+}
+
+RedisSearchCtx *SearchCtx_Refresh(RedisSearchCtx *sctx, RedisModuleString *keyName) {
+  // First we close the relevant keys we're touching
+  RedisModule_CloseKey(sctx->key);
+  RedisModuleCtx *redisCtx = sctx->redisCtx;
+  SearchCtx_Free(sctx);
+  // now release the global lock
+  RedisModule_ThreadSafeContextUnlock(redisCtx);
+  // try to acquire it again...
+  RedisModule_ThreadSafeContextLock(redisCtx);
+  // reopen the context - it might have gone away!
+  return NewSearchCtx(redisCtx, keyName);
+}
+
+RedisSearchCtx *NewSearchCtxDefault(RedisModuleCtx *ctx) {
+  RedisSearchCtx *sctx = rm_malloc(sizeof(*sctx));
+  *sctx = (RedisSearchCtx){.redisCtx = ctx};
+  return sctx;
+}
+
 void SearchCtx_Free(RedisSearchCtx *sctx) {
 
   rm_free(sctx);
@@ -268,7 +288,8 @@ InvertedIndex *Redis_OpenInvertedIndexEx(RedisSearchCtx *ctx, const char *term,
 }
 
 IndexReader *Redis_OpenReader(RedisSearchCtx *ctx, RSQueryTerm *term, DocTable *dt,
-                              int singleWordMode, t_fieldMask fieldMask, ConcurrentSearchCtx *csx) {
+                              int singleWordMode, t_fieldMask fieldMask, ConcurrentSearchCtx *csx,
+                              double weight) {
 
   RedisModuleString *termKey = fmtRedisTermKey(ctx, term->str, term->len);
   RedisModuleKey *k = RedisModule_OpenKey(ctx->redisCtx, termKey, REDISMODULE_READ);
@@ -282,7 +303,7 @@ IndexReader *Redis_OpenReader(RedisSearchCtx *ctx, RSQueryTerm *term, DocTable *
 
   InvertedIndex *idx = RedisModule_ModuleTypeGetValue(k);
 
-  IndexReader *ret = NewTermIndexReader(idx, dt, fieldMask, term);
+  IndexReader *ret = NewTermIndexReader(idx, dt, fieldMask, term, weight);
   if (csx) {
     ConcurrentSearch_AddKey(csx, k, REDISMODULE_READ, termKey, IndexReader_OnReopen, ret, NULL,
                             ConcurrentKey_SharedNothing);
@@ -354,20 +375,6 @@ int Redis_LoadDocumentEx(RedisSearchCtx *ctx, RedisModuleString *key, const char
   return REDISMODULE_OK;
 }
 
-Document *Redis_LoadDocuments(RedisSearchCtx *ctx, RedisModuleString **keys, int numKeys,
-                              const char **fields, int numFields, int *nump) {
-  Document *docs = calloc(numKeys, sizeof(Document));
-  int n = 0;
-
-  for (int i = 0; i < numKeys; i++) {
-    Redis_LoadDocumentEx(ctx, keys[i], fields, numFields, &docs[n], NULL);
-    docs[n++].docKey = keys[i];
-  }
-
-  *nump = n;
-  return docs;
-}
-
 int Redis_ScanKeys(RedisModuleCtx *ctx, const char *prefix, ScanFunc f, void *opaque) {
   long long ptr = 0;
 
@@ -454,11 +461,7 @@ int Redis_DropIndex(RedisSearchCtx *ctx, int deleteDocuments) {
   if (deleteDocuments) {
 
     DocTable *dt = &ctx->spec->docs;
-
-    for (size_t i = 1; i < dt->size; i++) {
-      const RSDocumentMetadata *dmd = dt->docs + i;
-      Redis_DeleteKey(ctx->redisCtx, DMD_CreateKeyString(dmd, ctx->redisCtx));
-    }
+    DocTable_ForEach(dt, Redis_DeleteKey(ctx->redisCtx, DMD_CreateKeyString(dmd, ctx->redisCtx)));
   }
 
   // Delete any dangling term keys
