@@ -3,7 +3,8 @@
 #include "extension.h"
 #include "util/minmax_heap.h"
 #include "ext/default.h"
-#include "util/array.h"
+#include "query_plan.h"
+#include "highlight.h"
 
 /*******************************************************************************************************************
  *  General Result Processor Helper functions
@@ -21,41 +22,13 @@ ResultProcessor *NewResultProcessor(ResultProcessor *upstream, void *privdata) {
   return p;
 }
 
-/* Safely call Next on an upstream processor, putting the result into res. If allowSwitching is 1,
- * we check the concurrent context and perhaps switch if needed.
- *
- * Note 1: Do not call processors' Next() directly, ONLY USE THIS FUNCTION
- *
- * Note 2: this function will not return RS_RESULT_QUEUED, but only OK or EOF. Any queued events
- * will be handled by this function
- * */
-inline int ResultProcessor_Next(ResultProcessor *rp, SearchResult *res, int allowSwitching) {
-  int rc;
-  ConcurrentSearchCtx *cxc = rp->ctx.qxc->conc;
-
-  do {
-
-    // If we can switch - we check the concurrent context switch BEFORE calling the upstream
-    if (allowSwitching) {
-      CONCURRENT_CTX_TICK(cxc);
-      // need to abort - return EOF
-      if (rp->ctx.qxc->state == QueryState_Aborted) {
-        return RS_RESULT_EOF;
-      }
-    }
-    rc = rp->Next(&rp->ctx, res);
-
-  } while (rc == RS_RESULT_QUEUED);
-  return rc;
-}
-
 /* Helper function - get the total from a processor, and if the Total callback is NULL, climb up
  * the
  * chain until we find a processor with a Total callback. This allows processors to avoid
  * implementing it if they have no calculations to add to Total (such as deeted/ignored results)
  * */
 size_t ResultProcessor_Total(ResultProcessor *rp) {
-  return rp->ctx.qxc->totalResults;
+  return rp->ctx.qxc ? rp->ctx.qxc->totalResults : 0;
 }
 
 /* Free a result processor - recursively freeing its upstream as well. If the processor does not
@@ -83,13 +56,18 @@ SearchResult *NewSearchResult() {
 
 /* Free the search result's internal data but not the result itself - it may be allocated on the
  * stack */
-inline void SearchResult_FreeInternal(SearchResult *r) {
+void SearchResult_FreeInternal(SearchResult *r) {
 
   if (!r) return;
   // This won't affect anything if the result is null
-  IndexResult_Free(r->indexResult);
-
-  RSFieldMap_Free(r->fields, 0);
+  if (r->indexResult) {
+    // IndexResult_Free(r->indexResult);
+    r->indexResult = NULL;
+  }
+  if (r->fields) {
+    RSFieldMap_Free(r->fields);
+    r->fields = NULL;
+  }
 }
 
 /* Free the search result object including the object itself */
@@ -121,7 +99,7 @@ int baseResultProcessor_Next(ResultProcessorCtx *ctx, SearchResult *res) {
     return RS_RESULT_EOF;
   }
   // if we've timed out - abort the root processor and return EOF
-  if (q->execCtx.state == QueryState_TimedOut) {
+  if (q->execCtx.state == QPState_TimedOut) {
     q->rootFilter->Abort(q->rootFilter->ctx);
     return RS_RESULT_EOF;
   }
@@ -159,10 +137,14 @@ int baseResultProcessor_Next(ResultProcessorCtx *ctx, SearchResult *res) {
 
   // the index result of the search result is not thread safe. It will be copied by the sorter later
   // on if we need it to be thread safe
-  res->indexResult = r;
+  res->indexResult = r;  // q->opts.needIndexResult ? r : NULL;
+
   res->score = 0;
-  res->sv = dmd->sortVector;
-  res->md = dmd;
+  res->sorterPrivateData = dmd->sortVector;
+  res->scorerPrivateData = dmd;
+  if (res->fields != NULL) {
+    res->fields->len = 0;
+  }
 
   return RS_RESULT_OK;
 }
@@ -199,7 +181,8 @@ int scorerProcessor_Next(ResultProcessorCtx *ctx, SearchResult *res) {
     if (RS_RESULT_EOF == (rc = ResultProcessor_Next(ctx->upstream, res, 0))) return rc;
 
     // Apply the scoring function
-    res->score = sc->scorer(&sc->scorerCtx, res->indexResult, res->md, ctx->qxc->minScore);
+    res->score =
+        sc->scorer(&sc->scorerCtx, res->indexResult, res->scorerPrivateData, ctx->qxc->minScore);
 
     // If we got the special score RS_SCORE_FILTEROUT - disregard the result and decrease the total
     // number of results (it's been increased by the upstream processor)
@@ -234,7 +217,7 @@ ResultProcessor *NewScorer(const char *scorer, ResultProcessor *upstream, RSSear
   sc->scorerFree = scx->ff;
   sc->scorerCtx.payload = req->payload;
   // Initialize scorer stats
-  IndexSpec_GetStats(req->sctx->spec, &sc->scorerCtx.indexStats);
+  IndexSpec_GetStats(upstream->ctx.qxc->sctx->spec, &sc->scorerCtx.indexStats);
 
   ResultProcessor *rp = NewResultProcessor(upstream, sc);
   rp->Next = scorerProcessor_Next;
@@ -264,10 +247,16 @@ ResultProcessor *NewScorer(const char *scorer, ResultProcessor *upstream, RSSear
  * finding the top N results
  ********************************************************************************************************************/
 
+typedef enum {
+  Sort_ByScore,
+  Sort_BySortKey,
+  Sort_ByFields,
+} SortMode;
 /* Sorter's private context */
 struct sorterCtx {
 
   // The desired size of the heap - top N results
+  // If set to 0 this is a growing heap
   uint32_t size;
 
   // The offset - used when popping result after we're done
@@ -288,15 +277,26 @@ struct sorterCtx {
   int accumulating;
 
   int saveIndexResults;
+
+  SortMode sortMode;
+};
+
+struct fieldCmpCtx {
+  RSMultiKey *keys;
+
+  // a bitmap where each bit corresponds to a variable in the keymap, specifying ascending (1) or
+  // descending (0)
+  uint64_t ascendMap;
 };
 
 /* Yield - pops the current top result from the heap */
 int sorter_Yield(struct sorterCtx *sc, SearchResult *r) {
 
-  // make sure we don't overshoot the heap size
-  if (sc->pq->count > 0 && sc->offset++ < sc->size) {
+  // make sure we don't overshoot the heap size, unless the heap size is dynamic
+  if (sc->pq->count > 0 && (!sc->size || sc->offset++ < sc->size)) {
     SearchResult *sr = mmh_pop_max(sc->pq);
     *r = *sr;
+    DMD_Decref(r->scorerPrivateData);
     free(sr);
     return RS_RESULT_OK;
   }
@@ -308,12 +308,31 @@ void sorter_Free(ResultProcessor *rp) {
   if (sc->pooledResult) {
     SearchResult_Free(sc->pooledResult);
   }
+  if (sc->cmpCtx) {
+    if (sc->sortMode == Sort_ByFields) {
+      struct fieldCmpCtx *fcc = sc->cmpCtx;
+      RSMultiKey_Free(fcc->keys);
+      free(fcc);
+    }
+  }
+
   // calling mmh_free will free all the remaining results in the heap, if any
   mmh_free(sc->pq);
   free(sc);
   free(rp);
 }
 
+static void keepResult(struct sorterCtx *sctx, SearchResult *r) {
+  DMD_Incref(r->scorerPrivateData);
+  if (sctx->sortMode == Sort_ByFields && r->fields) {
+    for (size_t ii = 0; ii < r->fields->len; ++ii) {
+      r->fields->fields[ii].val = RSValue_MakePersistent(r->fields->fields[ii].val);
+      r->fields->fields[ii].key = strdup(r->fields->fields[ii].key);
+      r->fields->isKeyAlloc = 1;
+    }
+  }
+}
+
 int sorter_Next(ResultProcessorCtx *ctx, SearchResult *r) {
   struct sorterCtx *sc = ctx->privdata;
   // if we're not accumulating anymore - yield the top result
@@ -323,7 +342,7 @@ int sorter_Next(ResultProcessorCtx *ctx, SearchResult *r) {
 
   if (sc->pooledResult == NULL) {
     sc->pooledResult = NewSearchResult();
-  } else {
+  } else if (sc->pooledResult->fields) {
     sc->pooledResult->fields->len = 0;
   }
   SearchResult *h = sc->pooledResult;
@@ -336,10 +355,13 @@ int sorter_Next(ResultProcessorCtx *ctx, SearchResult *r) {
   }
 
   // If the queue is not full - we just push the result into it
-  if (sc->pq->count + 1 < sc->pq->size) {
+  // If the pool size is 0 we always do that, letting the heap grow dynamically
+  if (!sc->size || sc->pq->count + 1 < sc->pq->size) {
 
     // copy the index result to make it thread safe - but only if it is pushed to the heap
     h->indexResult = NULL;
+
+    keepResult(sc, h);
     mmh_insert(sc->pq, h);
     sc->pooledResult = NULL;
     if (h->score < ctx->qxc->minScore) {
@@ -359,15 +381,17 @@ int sorter_Next(ResultProcessorCtx *ctx, SearchResult *r) {
     if (sc->cmp(h, minh, sc->cmpCtx) > 0) {
       // copy the index result to make it thread safe - but only if it is pushed to the heap
       h->indexResult = NULL;
-
       sc->pooledResult = mmh_pop_min(sc->pq);
-      sc->pooledResult->indexResult = NULL;
+      SearchResult_FreeInternal(sc->pooledResult);
+
+      keepResult(sc, h);
       mmh_insert(sc->pq, h);
     } else {
       // The current should not enter the pool, so just leave it as is
+      h->indexResult = NULL;
       sc->pooledResult = h;
       // make sure we will not try to free the index result of the pooled result at the end
-      sc->pooledResult->indexResult = NULL;
+      SearchResult_FreeInternal(sc->pooledResult);
     }
   }
 
@@ -390,24 +414,53 @@ static inline int cmpByScore(const void *e1, const void *e2, const void *udata)
 static int cmpBySortKey(const void *e1, const void *e2, const void *udata) {
   const RSSortingKey *sk = udata;
   const SearchResult *h1 = e1, *h2 = e2;
-  if (!h1->sv || !h2->sv) {
+  if (!h1->sorterPrivateData || !h2->sorterPrivateData) {
     return h1->docId < h2->docId ? -1 : 1;
   }
-  return -RSSortingVector_Cmp(h1->sv, h2->sv, (RSSortingKey *)sk);
+  return -RSSortingVector_Cmp(h1->sorterPrivateData, h2->sorterPrivateData, (RSSortingKey *)sk);
 }
 
-ResultProcessor *NewSorter(RSSortingKey *sk, uint32_t size, ResultProcessor *upstream,
-                           int copyIndexResults) {
+/* Compare results for the heap by sorting key */
+static int cmpByFields(const void *e1, const void *e2, const void *udata) {
+  const struct fieldCmpCtx *cc = udata;
+
+  const SearchResult *h1 = e1, *h2 = e2;
+  int ascending = 0;
+
+  for (size_t i = 0; i < cc->keys->len && i < sizeof(cc->ascendMap) * 8; i++) {
+    RSValue *v1 = RSFieldMap_GetByKey(h1->fields, &cc->keys->keys[i]);
+    RSValue *v2 = RSFieldMap_GetByKey(h2->fields, &cc->keys->keys[i]);
+    if (!v1 || !v2) {
+      break;
+    }
+
+    int rc = RSValue_Cmp(v1, v2);
+    // take the ascending bit for this property from the ascending bitmap
+    ascending = cc->ascendMap & (1 << i) ? 1 : 0;
+    if (rc != 0) return ascending ? -rc : rc;
+  }
+
+  int rc = h1->docId < h2->docId ? -1 : 1;
+  return ascending ? -rc : rc;
+}
+
+ResultProcessor *NewSorter(SortMode sortMode, void *sortCtx, uint32_t size,
+                           ResultProcessor *upstream, int copyIndexResults) {
 
   struct sorterCtx *sc = malloc(sizeof(*sc));
-
-  if (sk) {
-    sc->cmp = cmpBySortKey;
-    sc->cmpCtx = sk;
-  } else {
-    sc->cmp = cmpByScore;
-    sc->cmpCtx = NULL;
+  // select the sorting function by the sort mode
+  switch (sortMode) {
+    case Sort_ByScore:
+      sc->cmp = cmpByScore;
+      break;
+    case Sort_ByFields:
+      sc->cmp = cmpByFields;
+      break;
+    case Sort_BySortKey:
+      sc->cmp = cmpBySortKey;
+      break;
   }
+  sc->cmpCtx = sortCtx;
 
   sc->pq = mmh_init_with_size(size + 1, sc->cmp, sc->cmpCtx, SearchResult_Free);
   sc->size = size;
@@ -415,6 +468,7 @@ ResultProcessor *NewSorter(RSSortingKey *sk, uint32_t size, ResultProcessor *ups
   sc->pooledResult = NULL;
   sc->accumulating = 1;
   sc->saveIndexResults = copyIndexResults;
+  sc->sortMode = sortMode;
 
   ResultProcessor *rp = NewResultProcessor(upstream, sc);
   rp->Next = sorter_Next;
@@ -422,6 +476,15 @@ ResultProcessor *NewSorter(RSSortingKey *sk, uint32_t size, ResultProcessor *ups
   return rp;
 }
 
+ResultProcessor *NewSorterByFields(RSMultiKey *mk, uint64_t ascendingMap, uint32_t size,
+                                   ResultProcessor *upstream) {
+  struct fieldCmpCtx *c = malloc(sizeof(*c));
+  c->ascendMap = ascendingMap;
+  c->keys = mk;
+
+  return NewSorter(Sort_ByFields, c, size, upstream, 0);
+}
+
 /*******************************************************************************************************************
  *  Paging Processor
  *
@@ -453,13 +516,18 @@ int pager_Next(ResultProcessorCtx *ctx, SearchResult *r) {
   // not reached beginning of results
   if (pc->count < pc->offset) {
 
-    IndexResult_Free(r->indexResult);
-    free(r->fields);
+    // IndexResult_Free(r->indexResult);
+    RSFieldMap_Free(r->fields);
+    r->fields = NULL;
+
     pc->count++;
     return RS_RESULT_QUEUED;
   }
   // overshoot the count
   if (pc->count >= pc->limit + pc->offset) {
+    // IndexResult_Free(r->indexResult);
+    RSFieldMap_Free(r->fields);
+    r->fields = NULL;
     return RS_RESULT_EOF;
   }
   pc->count++;
@@ -472,6 +540,7 @@ ResultProcessor *NewPager(ResultProcessor *upstream, uint32_t offset, uint32_t l
   pc->offset = offset;
   pc->limit = limit;
   pc->count = 0;
+
   ResultProcessor *rp = NewResultProcessor(upstream, pc);
 
   rp->Next = pager_Next;
@@ -491,7 +560,9 @@ ResultProcessor *NewPager(ResultProcessor *upstream, uint32_t offset, uint32_t l
  *******************************************************************************************************************/
 struct loaderCtx {
   RedisSearchCtx *ctx;
-  FieldList *fields;
+  const char **fields;
+  size_t numFields;
+  int explicitReturn;
 };
 
 int loader_Next(ResultProcessorCtx *ctx, SearchResult *r) {
@@ -507,71 +578,82 @@ int loader_Next(ResultProcessorCtx *ctx, SearchResult *r) {
 
   // Current behavior skips entire result if document does not exist.
   // I'm unusre if that's intentional or an oversight.
-  RedisModuleString *idstr = DMD_CreateKeyString(r->md, lc->ctx->redisCtx);
-  if (!lc->fields->explicitReturn) {
+  RedisModuleString *idstr = DMD_CreateKeyString(r->scorerPrivateData, lc->ctx->redisCtx);
+  if (!lc->explicitReturn) {
     Redis_LoadDocument(lc->ctx, idstr, &doc);
   } else {
-    Array fieldList;
-    Array_Init(&fieldList);
-    for (size_t ii = 0; ii < lc->fields->numFields; ++ii) {
-      Array_Write(&fieldList, &lc->fields->fields[ii].name, sizeof(char *));
-    }
 
-    Redis_LoadDocumentEx(lc->ctx, idstr, (const char **)fieldList.data, lc->fields->numFields, &doc,
-                         &rkey);
+    Redis_LoadDocumentEx(lc->ctx, idstr, lc->fields, lc->numFields, &doc, &rkey);
     RedisModule_FreeString(lc->ctx->redisCtx, idstr);
-    Array_Free(&fieldList);
   }
   // TODO: load should return strings, not redis strings
   for (int i = 0; i < doc.numFields; i++) {
-    RSFieldMap_Add(&r->fields, doc.fields[i].name,
-                   doc.fields[i].text ? RS_RedisStringVal(doc.fields[i].text) : RS_NullVal());
+    if (doc.fields[i].text) {
+      RSFieldMap_Set(&r->fields, doc.fields[i].name, RS_RedisStringVal(doc.fields[i].text));
+    } else {
+      RSFieldMap_Set(&r->fields, doc.fields[i].name, RS_NullVal());
+    }
   }
   Document_Free(&doc);
 
   return RS_RESULT_OK;
 }
 
-ResultProcessor *NewLoader(ResultProcessor *upstream, RSSearchRequest *r) {
+void loader_Free(ResultProcessor *rp) {
+  struct loaderCtx *lc = rp->ctx.privdata;
+  free(lc->fields);
+  free(lc);
+  free(rp);
+}
+ResultProcessor *NewLoader(ResultProcessor *upstream, RedisSearchCtx *sctx, FieldList *fields) {
   struct loaderCtx *sc = malloc(sizeof(*sc));
-  sc->ctx = r->sctx;
-  sc->fields = &r->fields;
+
+  sc->ctx = sctx;
+  sc->fields = calloc(fields->numFields, sizeof(char *));
+  sc->numFields = fields->numFields;
+  for (size_t i = 0; i < fields->numFields; i++) {
+    sc->fields[i] = fields->fields[i].name;
+  }
+  sc->explicitReturn = fields->explicitReturn;
 
   ResultProcessor *rp = NewResultProcessor(upstream, sc);
 
   rp->Next = loader_Next;
-  rp->Free = ResultProcessor_GenericFree;
+  rp->Free = loader_Free;
   return rp;
 }
 
 /*******************************************************************************************************************
  * Building the processor chaing based on the processors available and the request parameters
  *******************************************************************************************************************/
-
-ResultProcessor *Query_BuildProcessorChain(QueryPlan *q, RSSearchRequest *req) {
-
+ResultProcessor *Query_BuildProcessorChain(QueryPlan *q, void *privdata, char **err) {
+  *err = NULL;
+  RSSearchRequest *req = privdata;
   // The base processor translates index results into search results
   ResultProcessor *next = NewBaseProcessor(q, &q->execCtx);
 
   // If we are not in SORTBY mode - add a scorer to the chain
-  if (req->sortBy == NULL) {
-    next = NewScorer(req->scorer, next, req);
+  if (q->opts.sortBy == NULL) {
+    next = NewScorer(q->opts.scorer, next, req);
+    // Scorers usually need the index results, let's tell the query plan that
+    q->opts.needIndexResult = 1;
   }
 
   // The sorter sorts the top-N results
-  next = NewSorter(req->sortBy, req->offset + req->num, next, req->fields.wantSummaries);
+  next = NewSorter(q->opts.sortBy ? Sort_BySortKey : Sort_ByScore, q->opts.sortBy,
+                   q->opts.offset + q->opts.num, next, req->opts.fields.wantSummaries);
 
   // The pager pages over the results of the sorter
-  next = NewPager(next, req->offset, req->num);
+  next = NewPager(next, q->opts.offset, q->opts.num);
 
   // The loader loads the documents from redis
   // If we do not need to return any fields - we do not need the loader in the loop
-  if (!(req->flags & Search_NoContent)) {
-    next = NewLoader(next, req);
-    if (req->fields.wantSummaries && (req->sctx->spec->flags & Index_StoreTermOffsets) != 0) {
+  if (!(q->opts.flags & Search_NoContent)) {
+    next = NewLoader(next, q->ctx, &req->opts.fields);
+    if (req->opts.fields.wantSummaries && (q->ctx->spec->flags & Index_StoreTermOffsets) != 0) {
       next = NewHighlightProcessor(next, req);
     }
   }
 
   return next;
-}
\ No newline at end of file
+}
