@@ -8,11 +8,11 @@
 #define L_TRACE 16
 
 
-int LOGGING_LEVEL; 
+extern int LOGGING_LEVEL; 
 //L_DEBUG | L_INFO
 
 
-void LOGGING_INIT(int level);
+extern void LOGGING_INIT(int level);
 
 #define LG_MSG(...) fprintf(stdout, __VA_ARGS__); 
 #define LG_DEBUG(...) if (LOGGING_LEVEL & L_DEBUG) { LG_MSG("[DEBUG %s:%d@%s] ", __FILE__ , __LINE__, __FUNCTION__); LG_MSG(__VA_ARGS__); LG_MSG("\n"); }
@@ -20,4 +20,4 @@ void LOGGING_INIT(int level);
 #define LG_WARN(...) if (LOGGING_LEVEL & L_WARN) { LG_MSG("[WARNING %s:%d] ", __FILE__ , __LINE__); LG_MSG(__VA_ARGS__); }
 #define LG_ERROR(...) if (LOGGING_LEVEL & L_ERROR) { LG_MSG("[ERROR %s:%d] ", __FILE__ , __LINE__); LG_MSG(__VA_ARGS__); }
 
-#endif
\ No newline at end of file
+#endif
