@@ -16,8 +16,15 @@ typedef struct mempool_t mempool_t;
 #else
 struct mempool_t;
 #endif
+
+#define MEMPOOOL_STATIC_ALLOCATOR(name, sz) \
+  void *name() {                            \
+    return malloc(sz);                      \
+  }
 /* Create a new memory pool */
 struct mempool_t *mempool_new(size_t cap, mempool_alloc_fn alloc, mempool_free_fn free);
+struct mempool_t *mempool_new_limited(size_t cap, size_t max_cap, mempool_alloc_fn alloc,
+                                      mempool_free_fn free);
 
 /* Get an entry from the pool, allocating a new instance if unavailable */
 void *mempool_get(struct mempool_t *p);
