@@ -680,7 +680,7 @@ void RSFieldMap_Reset(RSFieldMap *m) {
     for (size_t i = 0; i < m->len; i++) {
       RSValue_Free(m->fields[i].val);
       if (m->isKeyAlloc) {
-        free(m->fields[i].key);
+        free((void *)m->fields[i].key);
       }
     }
     m->isKeyAlloc = 0;
